# -*- coding: utf8 -*-
import re

from django.conf import settings
from django.contrib import messages
from django.shortcuts import render, redirect

from lib.access_auth import access_restricted
from lib.tools import confirmation

from mysql import connector
from mysql.connector.errors import ProgrammingError


class DatabaseSizeInfo(object):
    def __init__(self):
        self.table_count = 0
        self.total_size = 0


def _get_connection():
    return connector.connect(host=settings.MYSQL_HOST, user="root", password=settings.MYSQL_ROOT_PASSWORD)


def _get_db_size(db_name):
    conn = _get_connection()

    try:
        try:
            if re.match(r"^user_[a-z0-9]+$", db_name):
                cursor = conn.cursor()
                cursor.execute(f"USE {db_name}")
            else:
                raise Exception("Nom de base de donnée non valide.")
        except ProgrammingError as e:
            if e.errno == connector.errorcode.ER_NO_DB_ERROR or e.errno == connector.errorcode.ER_BAD_DB_ERROR:
                # Unknown database
                return None
            else:
                raise

        size_info = DatabaseSizeInfo()

        cursor.execute("SHOW TABLE STATUS;")
        for info in cursor.fetchall():
            size_info.table_count += 1
            size_info.total_size = info[6]
        cursor.close()

        return size_info

    finally:
        conn.close()


def _create_database(db_name, user, password):
    conn = _get_connection()

    try:
        cursor = conn.cursor()
        cursor.execute("START TRANSACTION;")
        cursor.execute(
            "CREATE USER IF NOT EXISTS '{}'@'%' IDENTIFIED BY '{}';".format(user, password.replace("'", "''"))
        )
        cursor.execute("CREATE DATABASE IF NOT EXISTS {};".format(db_name))
        cursor.execute("GRANT ALL ON {}.* TO '{}'@'%';".format(db_name, user))
        cursor.close()

    finally:
        conn.close()


def _change_db_password(user, password):
    conn = _get_connection()

    try:
        cursor = conn.cursor()
        cursor.execute("SET PASSWORD FOR {}@'%' = PASSWORD('{}');".format(user, password.replace("'", "''")))
        cursor.close()

    finally:
        conn.close()


def _drop_database(db_name, user):
    if _get_db_size(db_name) is None:
        return

    conn = _get_connection()

    try:
        cursor = conn.cursor()
        cursor.execute("START TRANSACTION;")
        cursor.execute("DROP DATABASE IF EXISTS {};".format(db_name))
        cursor.execute("DROP USER IF EXISTS '{}'@'%';".format(user))
        cursor.close()

    finally:
        conn.close()


@access_restricted
def accueil(request):
    nom_bdd = "user_" + request.user.username
    info_bdd = _get_db_size(nom_bdd)

    if request.method == "POST":
        if "creer_bdd" in request.POST and info_bdd is None:
            mdp = request.POST.get("mdp_bdd", "")
            if mdp:
                _create_database(nom_bdd, request.user.username, mdp)

                messages.success(request, "Votre base de données a été créée.")
            else:
                messages.error(request, "Veuillez entrer un mot de passe.")

            return redirect("gestion_bdd:accueil")

        elif "mod_mdp_bdd" in request.POST and info_bdd is not None:
            mdp = request.POST.get("mdp_bdd", "")
            if mdp:
                _change_db_password(request.user.username, mdp)

                messages.success(request, "Mot de passe modifié avec succès.")
            else:
                messages.error(request, "Veuillez entrer un mot de passe.")

            return redirect("gestion_bdd:accueil")

    if info_bdd is None:
        c = {"bdd_cree": False}
    else:
        c = {
            "bdd_creee": True,
            "nb_tables": info_bdd.table_count,
            "taille_donnees": info_bdd.total_size,
        }

    return render(request, "gestion_bdd/accueil.html", c)


@access_restricted
def suppression(request):
    nom_bdd = "user_" + request.user.username

    if request.method != "POST":
        return confirmation(
            request,
            "Êtes-vous sûr de vouloir supprimer votre base de données ? " "Cette opération ne pourra être annulée.",
            (("conf_annuler", "Annuler"), ("conf_ok", "Supprimer")),
        )

    if "conf_ok" not in request.POST:
        return redirect("gestion_bdd:accueil")

    _drop_database(nom_bdd, request.user.username)

    return redirect("gestion_bdd:accueil")
